#include <stdio.h>
#include <math.h>
#include "pgm.h"
#include "edges.h"
#include "cells.h"

extern unsigned char *image;
extern unsigned char *edge_map[ORIENTATIONS];

double *feature_map[ORIENTATIONS];
char *contrast_map;

main(int argc, char* argv[])
{
  int width, height, max;
  int i, j;
  int size;
  char file_name[]="block.small.pgm";

  if(argc==2)
    strcpy(file_name, argv[1]);

  max=read_pgm(file_name, &image, &width, &height);
  size=width*height;

  Edges(width, height, size);

  for (i=0; i<ORIENTATIONS; i++)
    feature_map[i]=(double *) calloc(4*size, sizeof(double));

  contrast_map=(char *) calloc(4*size,1);

  i=3;
  Cells(width, height, size, i);

  write_pgm("contrast.pgm", contrast_map, width, height, max);

}


void Cells(int width, int height, int size, int radius_center)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  int radius_surround, radius;
  static char file_name[]="t0.03.extra.pgm";
  float weight[ORIENTATIONS];
  double cell[ORIENTATIONS], max; 
  double **gaus_c, **gaus_s;

  radius_surround=radius_center*2+1;
  radius=radius_center+radius_surround;


  gaus_c=(double **) calloc(2*radius_center+1, sizeof(double*));
  for (i=0; i<2*radius_center+1; i++)
    gaus_c[i]=(double *) calloc(2*radius_center+1, sizeof(double));

  gaus_s=(double **) calloc(2*radius+1, sizeof(double*));
  for (i=0; i<2*radius+1; i++)
    gaus_s[i]=(double *) calloc(2*radius+1, sizeof(double));

  Calc_Weights(weight);

  Calc_Gaus(gaus_c, radius_center);
  Calc_Gaus(gaus_s, radius);

  for (pixel_y=radius; pixel_y<height-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<ORIENTATIONS; i++)
      {
	cell[i]=convolve(pixel_x, pixel_y, i, width,
			 radius_center, gaus_c);
	cell[i]=cell[i]-convolve(pixel_x, pixel_y, i, width, radius, 
			  	 gaus_s);


	temp=1;
      }

/*      for(i=0; i<ORIENTATIONS; i++)
/*	for(j=0; j<ORIENTATIONS; j++)
/*	  feature_map[i][pixel_y*width+pixel_x]+=weight[(j-i+4)%4]
/*							*(int)cell[j];
*/


      feature_map[0][pixel_y * width + pixel_x]=weight[0]*cell[0];
					       +weight[1]*cell[1]
					       +weight[2]*cell[2]
					       +weight[3]*cell[3];

      feature_map[1][pixel_y * width + pixel_x]=weight[3]*cell[1];
					       +weight[0]*cell[1]
					       +weight[1]*cell[2]
					       +weight[2]*cell[3];

      feature_map[2][pixel_y * width + pixel_x]=weight[2]*cell[2];
					       +weight[3]*cell[1]
					       +weight[0]*cell[2]
					       +weight[1]*cell[3];

      feature_map[3][pixel_y * width + pixel_x]=weight[1]*cell[0];
					       +weight[2]*cell[1]
					       +weight[3]*cell[2]
					       +weight[0]*cell[3];




    }/* pixels */

  for(i=0; i<ORIENTATIONS; i++)
  {
    max=Max_dbl(feature_map[i], size);
    Norm_dbl(feature_map[i], max, size);
  }

/*  Mult_Norm(feature_map, size);*/

  for(i=0; i<ORIENTATIONS; i++)
  {
    write_dbl_pgm(file_name, feature_map[i], width, height, 255);
    (file_name[1])++;
  }

}



void Calc_Weights(float *w)
{
  int i;
  float denom;

  w[0]=1.; 
  denom=ORIENTATIONS*ORIENTATIONS;
  if (ORIENTATIONS%2)
    denom=denom-1;

  for (i=1; i<=ORIENTATIONS/2; i++)
  {
    w[i]=(float)-4*i/denom;
    w[ORIENTATIONS-i]=w[i];
  }
}

    

void Calc_Gaus(double **gaus, int rad)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=exp(-(float)(i*i+j*j)/2.0);
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;

}




double convolve(int x, int y, int map, int width, int rad, double **gaus)
{
  int i,j;
  float temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
	temp+=(float) edge_map[map][(y-i)*width+(x-j)] * 
		gaus[i+rad][j+rad];

  return temp;
}



void Mult_Norm_dbl(double **array, int size)
{
  int i;
  double t_max, max=0;

  for(i=0; i<ORIENTATIONS; i++)
    if ((t_max=Max_dbl(array[i], size))>max)
      max=t_max;

  for(i=0; i<ORIENTATIONS; i++)
    Norm_dbl(array[i], max, size);

  return;
}




void Norm_dbl(double *array, double max, int size)
{
  int i;
  double mult_fac;

  if (max>0)
  {
    mult_fac=255./max;
    for(i=0; i<size; i++)
      array[i]=array[i]*mult_fac;
  }
  else
    for(i=0; i<size; i++)
      array[i]=0;
  return;
}
    


void Norm_char(char* map, int size)
{
  int i, j, max;
  float multFac;

  max=Max_char(map, size);

  if(max>0)
  {
    multFac=255.0/max;

    for(i=0; i<size; i++)
      map[i]=map[i]*multFac;
  }
}




int Max_int(int *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}

double Max_dbl(double *map, int size)
{
  int i;
  double max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}


int Max_char(char *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}




int write_dbl_pgm(char *file_name, double *map, int width, 
		  int height, int max)
{
  unsigned char c_map[height*width];
  int i;
  
  for(i=0; i<height*width; i++)
    c_map[i]=(unsigned char) map[i];

  write_pgm(file_name, c_map, width, height, max);
}
