/*
/*  -ed  display edge maps (default false)
/*  -fd  display feature maps (default false)
/*  -cd  display contrast map (default false)
/*  -es  save edge maps (default false)
/*  -fs  save feature maps (default false)
/*  -s center_radius surround_radius set radii for contrast cell (default 7&12)
 */


#include <stdio.h>
#include <math.h>
#include <sys/ddi.h>
#include "pgm.h"
#include "edges.h"
#include "cells.h"
#include "utils.h"


char *contrast_map;

main(int argc, char* argv[])
{
  unsigned char *image,
                *edge_map[ORIENTATIONS],
		*feature_map[ORIENTATIONS];
  double *dbl_map[ORIENTATIONS];
  int width, height, max;
  int i, j, k;
  int centre=7, surround=12;
  int size;
  int e_display=0, f_display=0, c_display=0;
  int e_save=0, f_save=0;
  char file_name[40];
  char c_file[40];
  char syscall[60];
  char s_num[3];
  int num;
  int s_ind;
  int success;

  
  /* parse command line */

  if(argc<2)
  {
    fprintf(stderr, "USAGE: contrast [-em][-fm][-cm][-es][-fs]\n");
    fprintf(stderr, "       [-s center_radius surround_radius]");
    fprintf(stderr, " filename\n");
    exit(-1);
  }
  
  else
    parse(argc, argv, &e_display, &f_display, &c_display,
	  &e_save, &f_save, &centre, &surround, file_name);


  /* Read image file */
  max=read_pgm(file_name, &image, &width, &height);
  size=width*height;
    
 
  /* find edges */
  Edges(width, height, size, edge_map, image);


  /* write and display edge maps */
  if (e_save || e_display)
    save_disp(e_display, e_save, file_name, edge_map, width, height,
		max, "edge");


  /* allocate memory for feature and contrast maps */
  for (i=0; i<ORIENTATIONS; i++)
  {
    dbl_map[i]=(double *) calloc(4*size, sizeof(double));
    feature_map[i]=(unsigned char *) calloc(4*size, 1);
  }
  contrast_map=(unsigned char *) calloc(4*size,1);
  

  /* calculate contrast values */
  success = Orientation(width, height, size, centre, surround, 
		  edge_map, dbl_map);


  /* write and display feature maps */
  if (f_save || f_display)
  {
    for(i=0; i<ORIENTATIONS; i++)
      dbl_to_char(feature_map[i], dbl_map[i], size);
    save_disp(f_display, f_save, file_name, feature_map, width, height, 
		max, "feature");
  }

  
  /* save contrast map */
  strcpy(c_file, file_name);
  s_ind=strlen(c_file)-3;
  c_file[s_ind]=0;
  num_to_str(centre, s_num);
  strcat(c_file, s_num);
  strcat(c_file, ".");
  num_to_str(surround, s_num);
  strcat(c_file, s_num);
  strcat(c_file, ".orient.pgm");
  write_pgm(c_file, contrast_map, width, height, max);

  /* display contrast map */
  if (c_display)
  {
    strcpy(syscall, "xv ");
    strcat(syscall, c_file);
    strcat(syscall, " &");
    system(syscall);
  }

}


int Orientation(int width, int height, int size, 
	   int radius_centre, int radius,
	   unsigned char **edge_map, double **dbl_map)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  static char file_name[]="c4.s06.o0.pgm",
	      contrast_name[]="c_map.c6.s06.pgm";
  float weight[ORIENTATIONS];
  double cell[ORIENTATIONS], max; 
  double **gaus_c, **gaus_s;

/*
  file_name[1]='0'+radius_centre;
  temp=radius;
  file_name[4]='0'+(temp/10);
  temp=temp%10;
  file_name[5]='0'+temp;
  file_name[8]='0';

  contrast_name[7]=file_name[1];
  contrast_name[10]=file_name[4];
  contrast_name[11]=file_name[5];
*/

  gaus_c=(double **) calloc(2*radius_centre+1, sizeof(double*));
  for (i=0; i<2*radius_centre+1; i++)
    gaus_c[i]=(double *) calloc(2*radius_centre+1, sizeof(double));

  gaus_s=(double **) calloc(2*radius+1, sizeof(double*));
  for (i=0; i<2*radius+1; i++)
    gaus_s[i]=(double *) calloc(2*radius+1, sizeof(double));

  Calc_Weights(weight);

  Calc_Gaus(gaus_c, radius_centre);
  Calc_Gaus(gaus_s, radius);

  for (pixel_y=radius; pixel_y<height-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<ORIENTATIONS; i++)
      {
	cell[i]=Convolve(pixel_x, pixel_y, i, width,
			 radius_centre, gaus_c, edge_map);
	cell[i]=cell[i]-Convolve(pixel_x, pixel_y, i, width, radius, 
			  	 gaus_s, edge_map);
      }

      for(i=0; i<ORIENTATIONS; i++)
	for(j=0; j<ORIENTATIONS; j++)
	  dbl_map[i][pixel_y*width+pixel_x]+=
				weight[(j-i+ORIENTATIONS)%ORIENTATIONS]*cell[j];


    }/* pixels */


  max=Mult_Norm_dbl(dbl_map, size);

  if (max > 0.0)
    Combine_Maps(dbl_map, size, contrast_map);

  return max;
}

/*
void Combine_Maps(double **dbl_map, int size, unsigned char *c_map)
{
  int i, j;
  double max=0.;

  for(i=0; i<size; i++)
  {
    for (j=0; j<ORIENTATIONS; j++)
      if (dbl_map[j][i]>max)
	max=dbl_map[j][i];
    c_map[i]=(char)max;
    max=0.;
  }

  return;
}



void Calc_Weights(float *w)
{
  int i;
  float denom;

  w[0]=1.; 
  denom=ORIENTATIONS*ORIENTATIONS;
  if (ORIENTATIONS%2)
    denom=denom-1;

  for (i=1; i<=ORIENTATIONS/2; i++)
  {
    w[i]=(float)-4*i/denom;
    w[ORIENTATIONS-i]=w[i];
  }
}

    
*
void Calc_Gaus(double **gaus, int rad)
{
  int i,j;
  double total=0.0;
  FILE *f_gaus;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/2.0);
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;

  f_gaus=fopen("gaus2", "w");

  for(i=-rad; i<=rad; i++)
  {
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
      fprintf(f_gaus, "%4d", (int)(gaus[i+rad][j+rad]*100000));
    }
    fprintf(f_gaus, "\n");
  }
}



/*
double Convolve(int x, int y, int map, int width, int rad, double **gaus,
		unsigned char **edge_map)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
	temp+=((double) edge_map[map][(y-i)*width+(x-j)]*1000000) * 
		gaus[i+rad][j+rad];

  return temp;
}



double Mult_Norm_dbl(double **array, int size)
{
  int i;
  double t_max, max=0;

  for(i=0; i<ORIENTATIONS; i++)
    if ((t_max=Max_dbl(array[i], size))>max)
      max=t_max;

  for(i=0; i<ORIENTATIONS; i++)
    Norm_dbl(array[i], max, size);

  return max;
}




void Norm_dbl(double *array, double max, int size)
{
  int i;
  double mult_fac;

  if (max>0)
  {
    mult_fac=255./max;
    for(i=0; i<size; i++)
      array[i]=array[i]*mult_fac;
  }
  else
    for(i=0; i<size; i++)
      array[i]=0;
  return;
}
    


void Norm_char(char* map, int size)
{
  int i, j, max;
  float multFac;

  max=Max_char(map, size);

  if(max>0)
  {
    multFac=255.0/max;

    for(i=0; i<size; i++)
      map[i]=map[i]*multFac;
  }
}




int Max_int(int *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}

double Max_dbl(double *map, int size)
{
  int i;
  double max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}


int Max_char(char *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}




int write_dbl_pgm(char *file_name, double *map, int width, 
		  int height, int max)
{
  unsigned char c_map[height*width];
  int i;
  
  for(i=0; i<height*width; i++)
  {
    if (map[i]>0.)
      c_map[i]=(unsigned char) map[i];
    else
      c_map[i]=0;
  }

  write_pgm(file_name, c_map, width, height, max);
}

void dbl_to_char(unsigned char *c_map, double *dbl_map, int size)
{
  int i;

  for (i=0; i<size; i++)
  {
    if(dbl_map > 0)
      c_map[i]=(unsigned char) dbl_map[i];
    else
      c_map[i]=0;
  }
}


void num_to_str(int num, char *str)
{
  int i=0, num2;

  num2=num;
  while(num2)
  {
    num2=num2/10;
    i++;
  }

  str[i--]=0;
  num2=num;
  while(num2)
  {
    str[i--]='0'+num2%10;
    num2=num2/10;
  }
}*/


