/*
/*  -cd  display contrast map (default false)
/*  -s center_radius surround_radius set radii for contrast cell (default 7&12)
 */


#include <stdio.h>
#include <math.h>
#include "pgm.h"
#include "intensity.h"
#include "utils.h"


main(int argc, char* argv[])
{
  unsigned char *image,
		*contrast_map;
  double *dbl_map;
  int width, height, max;
  int i, j, k;
  int centre=7, surround=12;
  int size;
  int c_display=0;
  char file_name[40];
  char c_file[40];
  char syscall[60];
  char s_num[3];
  int num;
  int s_ind;
  int success;

  
  /* parse command line */

  if(argc<2)
  {
    fprintf(stderr, "USAGE: contrast [-cd][-s center_radius surround_radius]");
    fprintf(stderr, " filename\n");
    exit(-1);
  }
  
  else
    parse_inten(argc, argv, &c_display, &centre, &surround, file_name);


  /* Read image file */
  max=read_pgm(file_name, &image, &width, &height);
  size=width*height;
    
 
  /* allocate memory for temp double map and contrast map */
  dbl_map=(double *) calloc(4*size, sizeof(double));
  contrast_map=(unsigned char *) calloc(4*size,1);
  

  /* calculate contrast values */
  success = Intensity(image, width, height, size, centre, surround, 
		      dbl_map, contrast_map);


  /* save contrast map */
  strcpy(c_file, file_name);
  s_ind=strlen(c_file)-3;
  c_file[s_ind]=0;
  num_to_str(centre, s_num);
  strcat(c_file, s_num);
  strcat(c_file, ".");
  num_to_str(surround, s_num);
  strcat(c_file, s_num);
  strcat(c_file, ".inten.pgm");
  write_pgm(c_file, contrast_map, width, height, max);


  /* display contrast map */
  if (c_display)
  {
    strcpy(syscall, "xv ");
    strcat(syscall, c_file);
    strcat(syscall, " &");
    system(syscall);
  }

}


int Intensity(unsigned char *im, int width, int height, int size, 
	      int radius_centre, int radius, double *dbl_map,
	      unsigned char *c_map)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  double max;
  double cell;
  double **gaus_c, **gaus_s;
  double save;
  FILE *f_ptr;
  char f_name[]="temp2";

  /* allocate memory for gaussians */
  gaus_c=(double **) calloc(2*radius_centre+1, sizeof(double*));
  for (i=0; i<2*radius_centre+1; i++)
    gaus_c[i]=(double *) calloc(2*radius_centre+1, sizeof(double));

  gaus_s=(double **) calloc(2*radius+1, sizeof(double*));
  for (i=0; i<2*radius+1; i++)
    gaus_s[i]=(double *) calloc(2*radius+1, sizeof(double));


  
  /* calculate values for gaussians */
    /*  center_surround(radius_centre, radius, gaus_c, gaus_s);*/
  Calc_Gaus(gaus_c, radius_centre);
  Calc_Gaus(gaus_s, radius);

  f_ptr=fopen(f_name, "w");

  /* convolve image with difference of gaussians */
  for (pixel_y=radius; pixel_y<height-radius; pixel_y++)
  {
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      cell=Convolve(pixel_x, pixel_y, width, radius_centre, gaus_c, im);
      if ((pixel_x<radius+5) && (pixel_y<radius+5))
	fprintf(f_ptr, "%g ", cell);

      cell=cell-Convolve(pixel_x, pixel_y, width, radius, gaus_s, im);
      if ((pixel_x<radius+5) && (pixel_y<radius+5))
	fprintf(f_ptr, "%g\n", cell);

      dbl_map[pixel_y*width+pixel_x]=fabs(cell);	

    }/* for pixel_x */

  }/* for pixel_y */

  fclose(f_ptr);

  /* normalize and convert to character */
  max=Max_dbl(dbl_map, size);
  Norm_dbl(dbl_map, max, size);
  max=Max_dbl(dbl_map, size);
  dbl_to_char(c_map, dbl_map, size);

  return max;
}


