#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "contrast.h"
#include "edges.h"


void norm_dbl(double *from, double *to, int size, double max)
{
  int i;
  double factor=255.0/max;
  FILE *fa, *fb;

  for(i=0; i<size; i++)
    to[i] = factor>0.0 ? from[i]*factor : 0.0;
}

void d_to_c(double *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i] = from[i]>0.0 ? (unsigned char)from[i] :0;
}


void contrast(int *inten_map, int *edge_map, int width, double **c_map,
		int c_radius, double *max, int *mag)
{
  int size = width * width;
  int s_radius=c_radius*3;
  unsigned char *char_map;
  double max_inten;
  double *orient, *inten;
  double **gaus_c, **gaus_s;
  double c_sig=1.3, s_sig=1.3;
  int i;

  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;

  inten = (double *) calloc(size, sizeof(double));
  orient = (double *) calloc(size, sizeof(double));
  *c_map = (double *) calloc(size, sizeof(double));


  /* allocate memory for gaussians */
  gaus_c=(double **) calloc(2*c_radius+1, sizeof(double*));
  for (i=0; i<2*c_radius+1; i++)
    gaus_c[i]=(double *) calloc(2*c_radius+1, sizeof(double));

  gaus_s=(double **) calloc(2*s_radius+1, sizeof(double*));
  for (i=0; i<2*s_radius+1; i++)
    gaus_s[i]=(double *) calloc(2*s_radius+1, sizeof(double));


  /* calculate values for gaussians */
  Calc_Gaus(gaus_c, c_radius, c_radius*c_sig);
  Calc_Gaus(gaus_s, s_radius, s_radius*s_sig);

  *max=Orientation(width, c_radius, s_radius, gaus_c, gaus_s, mag,
		edge_map, orient);
/*  max_inten=Intensity(width, c_radius, s_radius, gaus_c, gaus_s,
			inten_map, inten);
*/

  printf("Orientation max=%g\n", *max);
  printf("Intensity max=%g\n", max_inten);

/*  if (max_inten > *max) *max = max_inten;*/
  combine_maps(*max, *c_map, orient, inten, size);

} /* contrast()*/


void combine_maps(double max, double *values, double *o_contrast,
		  double *i_contrast, int size)
{
  int i, j;
  double tmax, factor;
  unsigned char *temp_c;
  int width=sqrt(size);
  static int level=0;
  char f_name[20];

  factor=255.0/max;

  printf("max=%g\n", max);

  for (i=0; i<size; i++)
  {
    tmax=i_contrast[i];
    if (o_contrast[i] > tmax)
      tmax=o_contrast[i];
  
    values[i] = (double) (tmax*factor);
  }

} /* combine_maps */



double Orientation(int width, int center_radius, int radius, 
		   double **gaus_c, double **gaus_s, int *mag,
		   int *edges, double *orient_contrast)
{
  int i, j, pixel_x, pixel_y;
  int temp, curr;
  double max=0.0, tmax=0.0;
  int f_map[(radius*2+1)*(radius*2+1)];

  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      curr=pixel_y*width+pixel_x;
      create_feature_map(pixel_x, pixel_y, width, radius, mag, edges,
			 f_map);

      orient_contrast[curr]=Convolve(radius, radius, radius*2+1,
				center_radius, gaus_c, f_map);
      orient_contrast[curr]-=Convolve(radius, radius, radius*2+1, 
				radius, gaus_s, f_map);

      if (orient_contrast[pixel_y*width+pixel_x]>max)
	max = orient_contrast[pixel_y*width+pixel_x];

    }/* for (pixels) */

  return max;

}/* Orientation()*/


double Intensity(int width, int center_radius, int radius, 
		 double **gaus_c, double **gaus_s,
		 int *values, double *inten_contrast)
{
  int i, j, pixel_x, pixel_y;
  double cell, t_cell;
  double max=0.0, tmax=0.0;

  int size=width*width;
  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;
  FILE *f_ptr;

  /* convolve image with difference of gaussians */
  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
  {
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      cell=Convolve(pixel_x, pixel_y, width, center_radius, gaus_c, values);
      cell=cell-Convolve(pixel_x, pixel_y, width, radius, gaus_s, values);
      inten_contrast[pixel_y*width+pixel_x]=fabs(cell);

      if(inten_contrast[pixel_y*width+pixel_x]>max)
	max = inten_contrast[pixel_y*width+pixel_x];

    }/* for pixel_x */
  } /* for(pixel_y) */

  return max;

} /* Intensity()*/



void Calc_Gaus(double **gaus, int rad, double sigma)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/(2.0*sigma));
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
}/* Calc_Gaus() */


double Convolve(int x, int y, int width, int rad, double **gaus,
                int *values)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
        temp+=(double) values[(y-i)*width+(x-j)] *
                gaus[i+rad][j+rad];

  return temp;
} /* Convolve */


void create_feature_map(int x, int y, int width, int rad, int *mag,
			int *edges, int *map)
{
  int i, j, diff;
  int focus=edges[y*width+x];
  double factor = 255.0/90.0;
  static int start=1;
  FILE *eds, *diffs, *feature, *out;

  if (focus != -1)
  {
/*
    if (start)
    {
      diffs=fopen("diff_data", "w");
      fprintf(diffs, "y=%d x=%d pos=%d focus=%d rad=%d\n", 
		y, x, y*width+x, focus, rad);
    }
*/
    x-=rad;
    y-=rad;

    for (i=0; i<rad*2+1; i++)
      for (j=0; j<rad*2+1; j++)
      {
        if ( edges[(i+y)*width+(j+x)] != -1 )
        {
	  diff=abs(focus-edges[(i+y)*width + (j+x)]);
      	  if (diff>90) diff=180-diff;
      	  map[i*(2*rad+1)+j]=(90-diff)*
			(double)mag[(i+y)*width +(j+x)]/255.0;
/*	  if(start)
	  { 
	    fprintf(diffs, "i=%d\tj=%d\t  edge=%d\t mag=%d\t diff=%d\t",
			i, j, edges[(i+y)*width + (j+x)], 
			mag[(i+y)*width +(j+x)]);
	    fprintf(diffs, "map=%d\n", map[i*(2*rad+1)+j]);
	  } /* if start *
*/
        } /* if edges */

        else map[i*(2*rad+1)+j]=0;

      } /* for i, j */
/*
    if (start)
    {
      fclose(diffs);
      start=0;
      eds=fopen("edge_data", "w");
      feature=fopen("feature_data", "w");
      fprintf(eds, "x=%d, y=%d\n", x, y);
      fprintf(feature, "x=%d, y=%d\n", x, y);
      for (i=0; i<rad*2+1; i++)
      {
      	for (j=0; j<rad*2+1; j++)
      	{
	  fprintf(eds, "%d ", edges[(i+y)*width + (j+x)]);
	  fprintf(feature, "%d ", map[i*(2*rad+1)+j]);
      	}
      	fprintf(eds, "\n");
      	fprintf(feature, "\n");
      } /* for i *
      fclose(eds);
      fclose(feature);
    } /* start *
*/
  } /* if focus */

  else
  {
    for (i=0; i<rad*2+1; i++)
      for (j=0; j<rad*2+1; j++)
        map[i*(2*rad+1)+j]=0;
  } /* else */
}
