#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "contrast.h"
#include "edges.h"
#include "globals.h"


void norm_dbl(double *from, double *to, int size, double max)
{
  int i;
  double factor=255.0/max;
  FILE *fa, *fb;

  for(i=0; i<size; i++)
    to[i] = factor>0.0 ? from[i]*factor : 0.0;
}

void d_to_c(double *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i] = from[i]>0.0 ? (unsigned char)from[i] :0;
}


void contrast(unsigned char **inten_map, unsigned char *edge_map[], 
		int width, double **c_map, int radius_c, double *max,
		int num_features)
{
  int size = width * width;
  int radius_s=radius_c*3;
  unsigned char *char_map;
  double max_inten;
  double **orient, **inten;
  double **gaus_c, **gaus_s;
  int i;

  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;

  inten = (double **) malloc(2 * sizeof(double *));
  for (i=0; i<2; i++)
  {
    inten[i] = (double *) calloc(size,  sizeof(double));
  }
  orient = (double **) malloc(Num_Orientations * sizeof(double *));
  for (i=0; i<Num_Orientations; i++)
  {
    orient[i] = (double *) calloc(size, sizeof(double));
  }
  *c_map = (double *) malloc(size * sizeof(double));


  /* allocate memory for gaussians */
  gaus_c=(double **) calloc(2*radius_c+1, sizeof(double*));
  for (i=0; i<2*radius_c+1; i++)
    gaus_c[i]=(double *) calloc(2*radius_c+1, sizeof(double));

  gaus_s=(double **) calloc(2*radius_s+1, sizeof(double*));
  for (i=0; i<2*radius_s+1; i++)
    gaus_s[i]=(double *) calloc(2*radius_s+1, sizeof(double));


  /* calculate values for gaussians */
  Calc_Gaus(gaus_c, radius_c, radius_c*Sigma_c);
  Calc_Gaus(gaus_s, radius_s, radius_s*Sigma_s);


  *max=Orientation(width, radius_c, radius_s, gaus_c, gaus_s, edge_map,
		  orient, Num_Orientations);

/*  *max=0; */

  max_inten=Orientation(width, radius_c, radius_s, gaus_c, gaus_s,
			inten_map, inten, 2);

  if (max_inten > *max) *max = max_inten;
  combine_maps(*max, *c_map, orient, inten, size);

/*
  sprintf(f_name, "inten.pgm");
  temp_c=(unsigned char *) malloc(size);
  temp_d=(double *) malloc(size * sizeof(double));
  norm_dbl(inten, temp_d, size, *max);
  d_to_c(temp_d, temp_c, size);
  write_pgm(f_name, temp_c, width, width, 255);

  for(i=0; i<Num_Orientations; i++)
  {
    sprintf(f_name, "orien%d.l%d.pgm", i, level);
    norm_dbl(orient[i], temp_d, size, max);
    d_to_c(temp_d, temp_c, size);
    write_pgm(f_name, temp_c, width, width, 255);
  }
  level++;
  free(temp_c);
  free(temp_d);
*/

} /* contrast()*/


void combine_maps(double max, double *values, double **o_contrast,
		  double **i_contrast, int size)
{
  int i, j;
  double tmax, factor;
  unsigned char *temp_c;
  int width=sqrt(size);
  static int level=0;
  char f_name[20];

  factor=255.0/max;

  printf("max=%g\n", max);

  for (i=0; i<size; i++)
  {
    tmax=0;
    for (j=0; j<2; j++)
      if (i_contrast[j][i] > tmax)
	tmax=i_contrast[j][i];
    for (j=0; j<Num_Orientations; j++)
      if (o_contrast[j][i] > tmax)
	tmax=o_contrast[j][i];

/*/
    tmax=0;
    if (o_contrast[1][i]>tmax)
	tmax=o_contrast[1][i];  
/*
    if (o_contrast[2][i]>tmax)
	tmax=o_contrast[2][i];  
*/

    values[i] = (double) (tmax*factor);
  }
} /* combine_maps */

void clear_maps()
{
  int i, j;

/*
  for (i=1; i<pyramid.numHierarchies; i++)
    for (j=0; j<pyramid.levelSize[0]; j++)
      data[i].level[0][0].value[j]=0.0;
*/
} /* clear_maps() */

double Orientation(int width, int center_radius, int radius, 
		   double **gaus_c, double **gaus_s,
		   unsigned char **edges, double **orient_contrast,
		   int num_features)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  float weight[num_features];
  double cell[num_features];
  double max=0.0;

  Calc_Weights(weight, num_features);

  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<num_features; i++)
      {
        cell[i]=WEIGHT*Convolve(pixel_x, pixel_y, width,
                         center_radius, gaus_c, edges[i]);
        cell[i]=cell[i]-Convolve(pixel_x, pixel_y, width, radius,
                                 gaus_s, edges[i]);
      }

      for(i=0; i<num_features; i++)
      {
        for(j=0; j<num_features; j++)
	{
          orient_contrast[i][pixel_y*width+pixel_x] 
			+= weight[(j-i+num_features)%num_features]
			   * cell[j];
	}
        if (orient_contrast[i][pixel_y*width+pixel_x]>max)
	  max=orient_contrast[i][pixel_y*width+pixel_x];
      } /* for (i) */

    }/* for (pixels) */

  return max;

}/* Orientation()*/

/*
double Intensity(int width, int center_radius, int radius, 
		 double **gaus_c, double **gaus_s,
		 unsigned char **values, double *inten_contrast,
		 int num_features)
{
  int i, j, pixel_x, pixel_y;
  double cell, t_cell;
  double max=0.0;

  int size=width*width;
  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;
  FILE *f_ptr;

/*  f_ptr=fopen("temp1", "w");
*/
  /* convolve image with difference of gaussians */
/*
  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
  {
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      cell=Convolve(pixel_x, pixel_y, width, center_radius, gaus_c,
			 values[0]);
      cell=cell-Convolve(pixel_x, pixel_y, width, radius, gaus_s, 
				 values[0]);
      inten_contrast[pixel_y*width+pixel_x]=fabs(cell);

      if(inten_contrast[pixel_y*width+pixel_x]>max)
	max = inten_contrast[pixel_y*width+pixel_x];

    }/* for pixel_x */
/*
  } /* for(pixel_y) */

/*
  sprintf(f_name, "values%d.pgm", level);
  temp_c=(unsigned char *) malloc(size);
  temp_d=(double *) malloc(size * sizeof(double));
  norm_dbl(inten_contrast, temp_d, size, max);
  f_ptr=fopen("temp1", "w");
  for (pixel_y=0; pixel_y<width; pixel_y++)
    for (pixel_x=0; pixel_x<width; pixel_x++)
      fprintf(f_ptr, "%d %d %g\n", pixel_y, pixel_x,
                temp_d[pixel_y*width+pixel_x]);
  fclose(f_ptr);
  d_to_c(temp_d, temp_c, size);
  write_pgm(f_name, values, width, width, 255);
  level++;
  free(temp_c);
  free(temp_d);
*/
/*
  return max;

} /* Intensity()*/



void Calc_Gaus(double **gaus, int rad, double sigma)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/(2.0*sigma));
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
}/* Calc_Gaus() */

void Calc_Weights(float *w, int num_features)
{
  int i;
  float denom;

  w[0]=1.0;
  w[1]=1.0;

  denom=num_features*num_features;
  if (num_features%2)
    denom=denom-1;
 

  for (i=1; i<=num_features/2; i++)
  {
    w[i]=(float)(-4*i)/denom;
    w[num_features-i]=w[i];
  }

} /* Calc_Weights() */


double Convolve(int x, int y, int width, int rad, double **gaus,
                unsigned char *values)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
        temp+=(double) values[(y-i)*width+(x-j)] *
                gaus[i+rad][j+rad];

  return temp;
} /* Convolve */














